<?php
	/**
	* Plugin Name: Fanplayr Conversions
	* Plugin URI: http://fanplayr.com/
	* Description: A plugin that adds Fanplayr Conversions to your WP e-Commerce shopping cart. See also: <a href="http://fanplayr.com" target="_blank">Fanplayr.com</a>
	* Version: 1.2.0
	* Author: Fanplayr Inc.
	* Author URI: http://fanplayr.com/
	**/

	define(FANPLAYR_VERSION, '1.2.0');

	// ------------------------------------------------------------------------------------------------------------------------------------
	// activate / deactivate

	/* Runs when plugin is activated */
	register_activation_hook(__FILE__,'fanplayr_wpc_install');

	/* Runs on plugin deactivation*/
	register_deactivation_hook( __FILE__, 'fanplayr_wpc_remove' );

	function fanplayr_wpc_install()
	{
		add_option("fanplayr_config_acckey", '', '', 'no');
	}

	function fanplayr_wpc_remove()
	{
		delete_option('fanplayr_config_acckey');
	}

	// ------------------------------------------------------------------------------------------------------------------------------------
	// helpers
	function fanplayr_wpc_jsonMessage($isError, $message, $extras = array())
	{
		global $wp_version;

		$extras['error'] = $isError;
		$extras['message'] = $message;
		$extras['version'] = FANPLAYR_VERSION;
		$extras['wordpress_version'] = $wp_version;

		return json_encode($extras);
	}

	// postVars as k/v array OR
	function fanplayr_wpc_httpGetContent($url, $vars = null)
	{
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1) ;

		if ($vars != null) {
			// WTF ?
			if (is_array($vars)) $vars = str_replace('&amp;', '&', http_build_query($vars));
			//curl_setopt($ch, CURLOPT_POSTFIELDS, $vars);
			//curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_URL, $url.'?'.$vars);
		}

		$r = curl_exec($ch);
		curl_close($ch);

		return $r;
	}

	// ------------------------------------------------------------------------------------------------------------------------------------

	class FanplayrSettingsPage
	{
	    private $options;

	    public function __construct()
	    {
	        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
	        add_action( 'admin_init', array( $this, 'page_init' ) );
	    }

	    /**
	     * Add options page
	     */
	    public function add_plugin_page()
	    {
				$fpLog = '<span id="fanplayr-social-coupons-menu-item" style="padding-left: 0px;"><div style="border-radius: 1px 1px 1px 1px; display: block; float: left; width: 16px; height: 16px; background-color: #f57611; margin-right: 5px;"><div style="color: rgb(255, 255, 255); font-weight: bold; font-family: Arial,Helvetica,sans-serif; margin-left: 5px; font-size: 12px;">F</div></div>Fanplayr</span>';
				add_options_page('Fanplayr Conversions', $fpLog, 'administrator', 'fanplayr', array( $this, 'create_admin_page' ));
	    }

	    public function create_admin_page()
	    {
	        // Set class property
	        $this->options = get_option( 'fanplayr-settings' );
	        ?>
	        <div class="wrap">
	            <h2>Fanplayr Settings</h2>
	            <form method="post" action="options.php">
	            <?php
	                // This prints out all hidden setting fields
	                settings_fields( 'my_option_group' );
	                do_settings_sections( 'my-setting-admin' );
	                submit_button();
	            ?>
	            </form>
	        </div>
	        <?php
	    }

	    /**
	     * Register and add settings
	     */
	    public function page_init()
	    {
	        register_setting(
	            'my_option_group', // Option group
	            'fanplayr-settings', // Option name
	            array( $this, 'sanitize' ) // Sanitize
	        );

	        add_settings_section(
	            'setting_section_id', // ID
	            'Connection settings', // Title
	            array( $this, 'print_section_info' ), // Callback
	            'my-setting-admin' // Page
	        );

	        add_settings_field(
	            'account_key', // ID
	            'Account Key', // Title
	            array( $this, 'fanplayr_account_key_callback' ), // Callback
	            'my-setting-admin', // Page
	            'setting_section_id' // Section
	        );

	    }

	    public function sanitize( $input )
	    {
	        $new_input = array();
	        if( isset( $input['account_key'] ) )
	            $new_input['account_key'] = sanitize_text_field( $input['account_key'] );

	        return $new_input;
	    }

	    public function print_section_info()
	    {
	        print '';
	    }

	    /**
	     * Get the settings option array and print one of its values
	     */
	    public function fanplayr_account_key_callback()
	    {
	        printf(
	            '<input type="text" id="account_key" name="fanplayr-settings[account_key]" value="%s" />',
	            isset( $this->options['account_key'] ) ? esc_attr( $this->options['account_key']) : ''
	        );
	    }
	}

	if( is_admin() )
	    $my_settings_page = new FanplayrSettingsPage();

	// ------------------------------------------------------------------------------------------------------------------------------------
	// get sessions working
	add_action('init', 'fanplayr_session_manager');
	function fanplayr_session_manager() {
		if (!session_id()) {
			session_start();
		}
	 }

	add_action('wp_logout', 'fanplayr_session_logout');
	function fanplayr_session_logout() {
		session_destroy();
	}

	// ------------------------------------------------------------------------------------------------------------------------------------
	// API
	// do this before print, but after other stuff
	add_action('init', 'fanplayr_wpc_init', 101);

	function fanplayr_wpc_init()
	{
		$dir = dirname(__FILE__);

		if (isset($_REQUEST['fanplayr']))
		{
			require_once('Fanplayr.class.php');

			$api = new FanplayrApi();

			if (isset($_REQUEST['coupon'])) {
				if (isset($_REQUEST['add'])) {
					$api->applyCoupon();
				} elseif (isset($_REQUEST['deputize'])) {
					$api->deputizeCoupon();
				}else{
					echo fanplayr_wpc_jsonMessage(true, 'Please use a valid method.');
				}
			}else if(isset($_REQUEST['compy'])) {
				if (isset($_REQUEST['setinstalldata'])) {
					$api->setInstallData();
				} elseif (isset($_REQUEST['joincomplete'])) {
					$api->joinComplete();
				} elseif (isset($_REQUEST['unlink'])) {
					$api->unlink();
				} elseif (isset($_REQUEST['getrules'])) {
					$api->getRules();
				} elseif (isset($_REQUEST['addwidget'])) {
					$api->addWidget();
				} elseif (isset($_REQUEST['removewidget'])) {
					$api->removeWidget();
				} elseif (isset($_REQUEST['getorders'])) {
					$api->getOrders();
				}else {
					echo fanplayr_wpc_jsonMessage(true, 'Please use a valid method.');
				}
			}else {
				echo fanplayr_wpc_jsonMessage(true, 'Please use a valid method.');
			}

			exit(1);
		}
	}

	// ------------------------------------------------------------------------------------------------------------------------------------
	// Print it out
	// do this after everything else
	add_action('wp_head', 'fanplayr_wpc_print', 100);

	function fanplayr_wpc_curPageURL()
	{
		$isHTTPS = (isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on");
		$port = (isset($_SERVER["SERVER_PORT"]) && ((!$isHTTPS && $_SERVER["SERVER_PORT"] != "80") || ($isHTTPS && $_SERVER["SERVER_PORT"] != "443")));
		$port = ($port) ? ':'.$_SERVER["SERVER_PORT"] : '';
		$url = ($isHTTPS ? 'https://' : 'http://').$_SERVER["SERVER_NAME"].$port.$_SERVER["REQUEST_URI"];
		return $url;
	}

	function fanplayr_wpc_print()
	{
		$_SESSION['fanplayr_last_url'] = fanplayr_wpc_curPageURL();

		try {
			$fanplayr_widget_keys = json_decode(get_option("fanplayr_config_widget_keys"));

			echo '<script>';
			if ($fanplayr_widget_keys){
				foreach($fanplayr_widget_keys as $k => $v) {
					?>window.fplr = (function(d, s, k) {var js = d.createElement(s), fjs = d.getElementsByTagName(s)[0], f = window.fplr || (f = {guid: 1, config: {}, _r: [], _s:[], ready: function(a) { f._r.push(a) }, show: function(a) { f._s.push(a) }}); js.async = true; js.src = 'http://site.fanplayr/website/' + k + '/?v=3'; fjs.parentNode.insertBefore(js, fjs); document.write('<div id="fplr-' + f.guid + '"></div>'); f.config[f.guid++] = {key: k}; return f; })(document, 'script', '<?=$v?>');<?
				}
			}
			echo '</script>';
		}catch(Exception $e){}
	}

	add_action('wp_footer', 'fanplayr_wpc_track');

	// genius tracking
	function fanplayr_wpc_track($content = '')
	{
		global $wpdb, $wpsc_cart, $post, $wpsc;

		$settings = get_option( 'fanplayr-settings' );
		$accountKey = array_key_exists('account_key', $settings) ? $settings['account_key'] : '';

		if (!$accountKey) {
			return;
		}

		// id of current page
		$atPage = $post->ID;

		$checkoutId = $wpdb->get_var( "SELECT `ID` FROM `" . $wpdb->posts . "` WHERE `post_content` LIKE '%[shoppingcart]%'  AND `post_type` = 'page' LIMIT 1" );
		$transResultId = $wpdb->get_var( "SELECT `ID` FROM `" . $wpdb->posts . "` WHERE `post_content` LIKE '%[transactionresults]%'  AND `post_type` = 'page' LIMIT 1" );

		$atCheckout = $atPage == $checkoutId;
		// actually not payment because it is the same page as the checkout
		// but instead it's the transaction results page
		$atPayment = $atPage == $transResultId;

		if (!$atPayment) {

			// enc data
			$lineItemCount = 0;
			$numItems = 0;
			$subtotal = 0;
			$discount = 0;
			$total = 0;
			$custEmail = '';
			$custId = '';
			$couponCode = '';
			$currencyIso = '';
			$productsJson = '';
			$productsA = array();

			// get current cart info
			$products = $wpsc_cart->cart_items;

			foreach($products as $p) {
				$lineItemCount++;
				$numItems += $p->quantity;
				$subtotal += $p->total_price;

				array_push($productsA, (object) array(
					'id' => $p->product_id,
					'qty' => $p->quantity,
					'name' => $p->product_name,
					'sku' => $p->sku,
					'price' => $p->unit_price,
					'url' => $p->product_url,
					'image' => $p->thumbnail_image->guid
				));
			}

			$productsJson = json_encode($productsA);

			$couponCode = $wpsc_cart->coupons_name;
			$discount = $wpsc_cart->coupons_amount;
			$total = $subtotal - $discount;
			if ($total < 0) $total = 0;

			// Get currency settings
			$currency_type = get_option( 'currency_type' );
			$wpsc_currency_data = $wpdb->get_row( $wpdb->prepare( "SELECT `symbol`, `symbol_html`, `code` FROM `" . WPSC_TABLE_CURRENCY_LIST . "` WHERE `id` = %d LIMIT 1", $currency_type ), ARRAY_A );
			$currencyIso = $wpsc_currency_data['code'];

			// not sure where / what email to get ...
			$custEmail = '';
			$custId = '';

			$productListPageId = $wpdb->get_var( "SELECT `ID` FROM `" . $wpdb->posts . "` WHERE `post_content` LIKE '%[productspage]%'  AND `post_type` = 'page' LIMIT 1" );

			// ----------------------------------------------------------------
			// page tracking ...

			$pageType = '';

			$productId = '';
			$productName = '';
			$productPrice = 0;
			$productSku = '';
			$productUrl = '';
			$productImage = '';

			$categoryId = '';
			$categoryName = '';

			if (is_home()) {
				$pageType = 'home';
			} else if ($productListPageId == $atPage) {
				$pageType = 'cat';
				$categoryId = '1';
			}else if ($_GET['wpsc-product']) {
				$pageType = 'prod';
				$productId = $_GET['wpsc-product'];
			}else if ($atCheckout) {
				$pageType = 'cart';
			}else {
				$pageType = 'page';
			}

			$cartAction = 'override';
			/*$cartAction = 'repeat';
			if ($pageType == 'cart') {
				$cartAction = 'override';
			}*/

			// ----------------------------------------------------------------

			// URIs for adding to cart and deputizing
			$shopUrl = parse_url(get_option('siteurl') . '/');
			$storeDomain = $shopUrl['host'];
			$atcUri = $shopUrl . '?fanplayr&coupon&add&code=%c';
			$depUri = $shopUrl . '?fanplayr&coupon&deputize&a=%a&d=%d';

	    $output = <<<EOT
				<script>
				(function(d, w, s) {
				  var f = w.fanplayr = w.fanplayr || { _i:[] };
				  f._i.push({
				    type: 'st',
				    accountKey: '$accountKey',
				    storeDomain: '$storeDomain',
				    data: {
				      cartAction: '$cartAction',
				      lineItemCount: $lineItemCount,
				      numItems: $numItems,
				      subTotal: $subtotal,
				      total: $total,
				      discount: $discount,
				      couponCode: '$couponCode',
				      pageType: '$pageType',
				      categoryId: '$categoryId',
				      categoryName: '$categoryName',
				      productId: '$productId',
				      productName: '$productName',
				      productSku: '$productSku',
				      productPrice: $productPrice,
				      productUrl: '$productUrl',
				      productImage: '$productImage',
				      currency: '$currencyIso',
				      customerEmail: '$custEmail',
				      customerId: '$custId',
				      customerSegment: '',
				      customerGroup: '',
				      products : '$productsJson',
				      shopType: 'custom',
				      version: 3
				    }
				  });
				  var js = d.createElement(s);
				  var fjs = d.getElementsByTagName(s)[0];
				  js.async = true;
				  js.src = 'https://d1q7pknmpq2wkm.cloudfront.net/js/my.fanplayr.com/fp_smart.js?' + (new Date().getTime());
				  fjs.parentNode.insertBefore(js, fjs);
				})(document, window, 'script');
				</script>
EOT;

//echo '<pre>'.htmlentities($output).'</pre>';

			echo $output;

		}

		// deputization
		if ($atCheckout) {
			$deputizationEmbed = '';
			// needs to be done on all pages because we see the coupon in the
			// cart summary
			@$deputies = $_SESSION['fanplayr_deputy_codes'];
			if (is_array($deputies))
			{
				ob_start();
				require('deputy_tpl.php');
				$deputizationEmbed = ob_get_contents();
				ob_end_clean();
			}
			echo $deputizationEmbed;
		}
	}

	// -------------------------------------------------------------------------------------------------------------------------------------
	// Short codes
	add_filter( 'the_content', 'fanplayr_wpc_transaction_results', 13 );

	function fanplayr_decode_session_cookie( $cookie_name ) {
		if ( !empty($_COOKIE[$cookie_name]) ) {
			$data   = array();
			$params = explode(';', $_COOKIE[$cookie_name]);

			foreach ( $params as $param ) {
				$parts = explode('=', $param);
				$type  = substr($parts[1], 0, 1);
				$value = substr($parts[1], 1);

				if ( empty($type) ) continue;

				if ( $type === 'n' || $type === 'u' ) { $value = null; }
				if ( $type === 'b' )                  { $value = (intval($value, 10) !== 0); }
				if ( $type === 'f' )                  { $value = floatval($value); }
				if ( $type === 's' )                  { $value = urldecode($value); }

				$data[urldecode($parts[0])] = $value;
			}

			return $data;
		}
		return null;
	}

	function fanplayr_wpc_transaction_results($content = '')
	{
		global $wpdb, $post;

		$settings = get_option( 'fanplayr-settings' );
		$accountKey = array_key_exists('account_key', $settings) ? $settings['account_key'] : '';

		if (!$accountKey) {
			return;
		}

		$postId = $wpdb->get_var( "SELECT id FROM `" . $wpdb->posts . "` WHERE `post_content` LIKE '%[transactionresults]%'  AND `post_type` = 'page' LIMIT 1" );

		//if ( preg_match( "/\[fanplayr_track\]/", $content ) )
		if ( (int)$postId == (int)$post->ID )
		{
			$output = '';
			$sessionId = $_GET['sessionid'];
			$purchaseLog = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM `" . WPSC_TABLE_PURCHASE_LOGS . "` WHERE `sessionid`= %s LIMIT 1", $sessionId), ARRAY_A );

			$email = wpsc_get_buyers_email($purchaseLog['id']);

			$currencyCode = $wpdb->get_results("SELECT `code` FROM `".WPSC_TABLE_CURRENCY_LIST."` WHERE `id`='".get_option('currency_type')."' LIMIT 1",ARRAY_A);
			$localCurrencyCode = $currencyCode[0]['code'];

			$orderNumber = $purchaseLog['id'];
			$orderEmail = $email;
			$orderDate = gmdate("Y-m-d H:i:s", $purchaseLog['date']);
			$currency = $localCurrencyCode;
			$net = (float)$purchaseLog['totalprice'];
			$discount = (float)$purchaseLog['discount_value'];
			$gross = $net + $discount;

			$firstName = fanplayr_wpc_get_checkout_field($purchaseLog['id'], 'billingfirstname');
			$lastName = fanplayr_wpc_get_checkout_field($purchaseLog['id'], 'billinglastname');
			$customerEmail = $email;
			//$customerId = $order->getCustomerId();
			$discountCode = $purchaseLog['discount_data'];
			if (!$discountCode) $discountCode = '';

			$shipping = 0;
			$tax = 0;

			$shopUrl = parse_url(get_option('siteurl') . '/');
			$storeDomain = $shopUrl['host'];

			$productsJson = '';

			// add session and user keys
			$sessionVars = fanplayr_decode_session_cookie('fplr_st');
			if ($sessionVars) {
				@$vars .= '&sessionKey=' . $sessionVars['key'];
				@$vars .= '&userKey=' . $sessionVars['user'];
			}

	    $output = <<<EOT
				<script>
					(function(d, w, s) {
					  if ( !w.fp_sales_orders ) {
					    w.fp_sales_orders = {
					      accountKey: '$accountKey',
					      data: {
					      	cartAction: 'repeat',
					        orderId: '$orderNumber',
					        orderNumber: '$orderNumber',
					        orderDate: '$orderDate',
					        subTotal: $gross,
					        total: $net,
					        discount: $discount,
					        discountCode: '$discountCode',
					        currency: '$currency',
					        orderEmail: '$customerEmail',
					        firstName: '$firstName',
					        lastName: '$lastName',
					        customerEmail: '$customerEmail',
					        customerId: '',
					        shipping: $shipping,
					        tax: $tax,
					        shopType: 'custom',
					        version: 3,
					        products: '$productsJson',
					        storeDomain: '$storeDomain'
					      }
					    };
					    var js = d.createElement(s);
					    var fjs = d.getElementsByTagName(s)[0];
					    js.async = true;
					    js.src = 'https://d1q7pknmpq2wkm.cloudfront.net/js/my.fanplayr.com/fp_custom_orders.js?' + (new Date().getTime());
					    fjs.parentNode.insertBefore(js, fjs);
					  }
					})(document, window, 'script');
				</script>
EOT;
//echo '<pre>'.htmlentities($output).'</pre>';
			return $content . $output;
		} else {
			return $content;
		}
	}

	function fanplayr_wpc_get_checkout_field($purchaseId, $field)
	{
		global $wpdb;
		$formField = $wpdb->get_results( "SELECT `id`,`type` FROM `" . WPSC_TABLE_CHECKOUT_FORMS . "` WHERE `unique_name` IN ('" . $field . "') AND `active` = '1' ORDER BY `checkout_order` ASC LIMIT 1", ARRAY_A );
		return $wpdb->get_var( $wpdb->prepare( "SELECT `value` FROM `" . WPSC_TABLE_SUBMITED_FORM_DATA . "` WHERE `log_id` = %d AND `form_id` = '" . $formField[0]['id'] . "' LIMIT 1", $purchaseId ) );
	}
?>